@@ -26,9 +26,9 @@ XAS::Class - A Perl extension for the XAS environment
 =head1 DESCRIPTION
 
 This module ties the XAS environment to the base Badger object framework. It
-exposes the defined constants and utilities that reside in XAS::Constants and
-XAS::Utils. Which inherits from L<Badger::Constants|Badger::Constants> and 
-L<Badger::Utils|Badger::Utils>.
+exposes the defined constants and utilities that reside in L<XAS::Constants|XAS::Constants> and
+L<XAS::Utils|XAS::Utils>. Which inherits from L<Badger::Constants|http://badgerpower.com/docs/Badger/Class.html> and 
+L<Badger::Utils|http://badgerpower.com/docs/Badger/Utils.html>.
 
 =head1 SEE ALSO
 
@@ -44,10 +44,12 @@ Kevin L. Esteb, E<lt>kevin@kesteb.usE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012 by Kevin L. Esteb
+Copyright (C) 2014 Kevin L. Esteb
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
 at your option, any later version of Perl 5 you may have available.
 
+See L<http://dev.perl.org/licenses/> for more information.
+
 =cut
